package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.integrations.SoresClient;
import dk.nsp.idsas.personinformation.PersonInformationClient;
import dk.nsp.idsas.personinformation.PersonInformationClientException;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;
import org.springframework.beans.factory.annotation.Autowired;

import javax.sql.DataSource;
import java.util.GregorianCalendar;

public class BlurringService {

    @Autowired
    private DataSource dataSource;

    @Autowired
    private AuditLogger auditLogger;

    @Autowired
    private PersonInformationClient personInformationClient;

    public void createBlurring(IdsasActor actor, String idClassification, String cpr, GregorianCalendar endDateTime) throws IdsasServiceException {

        // Tjek at CVR er whitelisted til at kalde createBlurring
        WhitelistDao whitelistDao = new WhitelistDao(dataSource);
        boolean isWhiteListed = whitelistDao.isWhitelisted(actor.getCvr(), WhitelistType.BLURRING);
        if (!isWhiteListed) {
            throw new IdsasServiceException(ErrorMessages.whitelistError(actor.getCvr(), WhitelistType.BLURRING), true);
        }

        // Tjek at CVR mapper til en SOR kode (via SORES)
        SoresClient soresClient = new SoresClient();
        boolean cvrMapsToSor = soresClient.cvrMapsToSor(actor.getCvr());
        if (!cvrMapsToSor) {
            throw new IdsasServiceException(ErrorMessages.cvrDoesNotMapToSorError(actor.getCvr()), true);
        }

        // Tjek at patientId er gyldig (via "personinformation")
        boolean patientIdIsValid = false;
        try {
            patientIdIsValid = personInformationClient.personExists(cpr);
            if (!patientIdIsValid) {
                throw new IdsasServiceException(ErrorMessages.cprIsNotValidError(cpr), true);
            }
        } catch (PersonInformationClientException e) {
            throw new IdsasServiceException(e.getMessage(), false);
        }

        // Entity oprettes og gemmes
        BlurringEntity entity = new BlurringEntity(
                cpr,
                idClassification,
                actor.getCvr(),
                "CVR",
                endDateTime.toZonedDateTime().toInstant());

        BlurringDao blurringDao = new BlurringDao(dataSource);
        blurringDao.insert(entity);

        // Audit logning
        auditLogger.logCreateBlurring(cpr, actor.getCvr());
    }
}
