package dk.nsp.idsas.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.personinformation.PersonInformationClient;
import dk.nsp.idsas.personinformation.PersonInformationClientException;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.AuditLogger;
import org.mockito.Mockito;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.sql.DataSource;

import static org.mockito.ArgumentMatchers.anyString;

@Configuration
public class ServiceTestConfiguration {
    @Bean
    public BlurringService getBlurringService() {
        return new BlurringService();
    }

    @Bean
    public BlurringDao getBlurringDao(DataSource dataSource) {
        return new BlurringDao(dataSource);
    }

    @Bean
    public LookupService getLookupService(BlurringDao blurringDao, AuditLogger auditLogger, String allowedClientSystemName) {
        return new LookupService(blurringDao, auditLogger, allowedClientSystemName);
    }

    @Bean
    public AuditLogger auditLogger() {
        return new AuditLogger();
    }

    @Bean(name="allowedClientSystemName")
    public String getAllowedClientSystemName() {
        return "TestSystem";
    }

    @Bean
    public PersonInformationClient personInformationClient()  {
        PersonInformationClient client = Mockito.mock(PersonInformationClient.class);
        try {
            client = Mockito.when(client.personExists(anyString())).thenReturn(true).getMock();
        } catch (PersonInformationClientException e) {
            throw new RuntimeException(e);
        }
        return client;
    }
}
